#include "stdinc.h"
#include ".\game.h"

#include "colors.h"
#include "Screen.h"
#include "Well.h"
#include "font.h"
#include "controller.h"
#include "Piece.h"
#include "random.h"

#define PIECE_SHIFT_DELAY 6

int iPieceShiftDelay;
bool g_bSwappedThisTurn;

Piece g_oNextPiece[ NEXT_PIECE_COUNT ];
int g_iScore;
int g_iLevel;
int g_iLinesCleared;

int g_iFrameCtr;
enum GameMode g_eGameMode;
int g_iFallSpeed;

const int k_ScoreLinesCleared[] =
{
    1,
    10,
    30,
    70,
    200
};

#define LINES_PER_LEVEL 12

#define GAME_PIECE_DROP_FRAME_DELAY 30

#define COLLISION_FLAG_LEFT     (1<<0)
#define COLLISION_FLAG_RIGHT    (1<<1)
#define COLLISION_FLAG_BOTTOM   (1<<2)
#define COLLISION_FLAG_PIECE    (1<<3)

#define NEXT_PIECE_X            30
#define NEXT_PIECE_Y            (30)

bool g_bQuitApp;
Piece g_oGamePiece;
int g_iGamePieceX;
int g_iGamePieceY;
int g_iGamePieceDropDelay;
int g_iCollisionFlags;

void Game_Init()
{
    int i;

    g_iFallSpeed = GAME_PIECE_DROP_FRAME_DELAY;
    g_iGamePieceDropDelay = g_iFallSpeed;
    g_bSwappedThisTurn = false;

    Controller_Init();
    Screen_Init();
    Well_Init();
    g_bQuitApp = false;

    for( i = 0; i < NEXT_PIECE_COUNT; i++ )
    {
        g_oNextPiece[i].fScreenX = g_oNextPiece[i].fTargetScreenX = itof( NEXT_PIECE_X );
        g_oNextPiece[i].fScreenY = g_oNextPiece[i].fTargetScreenY = itof( NEXT_PIECE_Y + i * 60 );
    }

    g_iScore = 0;
    g_iLevel = 0;
    g_iLinesCleared = 0;

    Game_CreatePiece();
    g_eGameMode = GAME_MODE_WAIT;
    g_iFrameCtr++;
}

void Game_CreatePiece()
{
    int i;

    PieceCreate( &g_oGamePiece );
    g_iGamePieceX = WELL_WIDTH >> 1;
    g_iGamePieceY = -2;

    Game_UpdateGamePieceScreenPos();
    g_oGamePiece.fScreenX = g_oGamePiece.fTargetScreenX;
    g_oGamePiece.fScreenY = g_oGamePiece.fTargetScreenY;

    // Create the next peice
    int iSeed = RandGetSeed();
    for( i = 0; i < NEXT_PIECE_COUNT; i++ )
    {
        PieceCreate( &g_oNextPiece[ i ] );
    }
    RandSetSeed( iSeed );

    iPieceShiftDelay = 0;
}

void Game_Deinit()
{
    Well_Deinit();
}

void Game_Update()
{
    Controller_Update();

    int iPressed = Controller_KeysPressed();
    int iDown = Controller_KeysDown();

    switch( g_eGameMode )
    {
    case GAME_MODE_PAUSED:
        if( iPressed & (CTRL_START|CTRL_CROSS|CTRL_CIRCLE) )
        {
            g_eGameMode = GAME_MODE_PLAYING;
        }
        break;
    case GAME_MODE_WAIT:
        if( iPressed & (CTRL_START|CTRL_CROSS|CTRL_CIRCLE) )
        {
            g_eGameMode = GAME_MODE_PLAYING;
            Well_Init();
            RandSetSeed( g_iFrameCtr );
            Game_CreatePiece();

            g_iScore = 0;
            g_iLevel = 0;
            g_iFallSpeed = GAME_PIECE_DROP_FRAME_DELAY;
            g_bSwappedThisTurn = false;
        }
        //else if( iPressed & CTRL_TRIANGLE )
        //{
        //    g_bQuitApp = true;
        //}
        break;
    case GAME_MODE_PLAYING:
        if( iPieceShiftDelay > 0 )
        {
            iPieceShiftDelay--;
        }

        if( iPressed & CTRL_START )
        {
            g_eGameMode = GAME_MODE_PAUSED;
            break;
        }

        if( !g_bSwappedThisTurn && (iPressed & (CTRL_LTRIGGER|CTRL_RTRIGGER) ) )
        {
            g_bSwappedThisTurn = true;
            Game_CreatePiece();
            break;
        }

        if( iDown & CTRL_DOWN )
        {
            g_iGamePieceDropDelay -= g_iFallSpeed;
        }

        if( !iPieceShiftDelay )
        {
            if( iDown & CTRL_LEFT )
            {
                Game_GamePieceLeft();
                Game_UpdateGamePieceScreenPos();
                iPieceShiftDelay = PIECE_SHIFT_DELAY;
            }

            if( iDown & CTRL_RIGHT )
            {
                Game_GamePieceRight();
                Game_UpdateGamePieceScreenPos();
                iPieceShiftDelay = PIECE_SHIFT_DELAY;
            }
        }

        if( iPressed & (CTRL_TRIANGLE|CTRL_CIRCLE) )
        {
            Game_PieceRotateClockwise();
        }

        if( iPressed & (CTRL_SQUARE|CTRL_CROSS) )
        {
            Game_PieceRotateCounterClockwise();
        }

        if( g_iGamePieceDropDelay-- < 0 )
        {
            g_iGamePieceDropDelay = g_iFallSpeed;
            if( Game_DropGamePiece() )
            {
                Game_UpdateGamePieceScreenPos();
            }
            else
            {
                Game_ApplyPiece();
                int iCleared = Well_CheckForCompleteLines();
                Game_CreatePiece();
                g_bSwappedThisTurn = false;

                if( !Game_TestGamePiece( g_iGamePieceX, g_iGamePieceY ) )
                {
                    g_eGameMode = GAME_MODE_WAIT;
                }

                g_iScore += k_ScoreLinesCleared[ iCleared ];
                g_iLinesCleared += iCleared;
                Game_UpdateLevel();
            }
        }

        Well_Update();

        PieceUpdate( &g_oGamePiece );
        break;
    }
    g_iFrameCtr++;
}

void Game_UpdateLevel()
{
    int iNextLevel = g_iLevel + 1;
    int iLinesToNextLevel = iNextLevel * LINES_PER_LEVEL;

    if( g_iLinesCleared >= iLinesToNextLevel )
    {
        g_iLevel++;
        g_iFallSpeed = GAME_PIECE_DROP_FRAME_DELAY - 3 * g_iLevel;
        if( g_iFallSpeed < 4 ) {
            g_iFallSpeed = 4;
        }
    }

}

void Game_ApplyPiece()
{
    int x, y;

    for( x = 0; x < 4; x++ )
    {
        for( y = 0; y < 4; y++ )
        {
            if( g_oGamePiece.aMap[x][y] )
            {
                Cell* pCell = &g_aWell[ g_iGamePieceX + x ][ g_iGamePieceY + y ];           
                pCell->bOccupied = true;
                pCell->iColor = g_oGamePiece.iColor;
            }
        }
    }
}

bool Game_DropGamePiece()
{
    if( Game_TestGamePiece( g_iGamePieceX, g_iGamePieceY + 1 ) )
    {
        g_iGamePieceY++;
        return true;
    }
    else
    {
        return false;
    }
}

bool Game_GamePieceRight()
{
    if( Game_TestGamePiece( g_iGamePieceX + 1, g_iGamePieceY ) )
    {
        g_iGamePieceX++;
        return true;
    }
    else
    {
        return false;
    }
}

bool Game_GamePieceLeft()
{
    if( Game_TestGamePiece( g_iGamePieceX - 1, g_iGamePieceY ) )
    {
        g_iGamePieceX--;
        return true;
    }
    else
    {
        return false;
    }
}

bool Game_PieceRotateClockwise()
{
    int iPieceX = g_iGamePieceX;
    int iPieceY = g_iGamePieceY;
    PieceRotateClockwise( &g_oGamePiece );

    int iMoveCnt = 3;
    while( iMoveCnt-- )
    {
        if( Game_TestGamePiece( iPieceX, iPieceY ) )
        {
            if( iPieceX != g_iGamePieceX || iPieceY != g_iGamePieceY )
            {
                g_iGamePieceX = iPieceX;
                g_iGamePieceY = iPieceY;
                Game_UpdateGamePieceScreenPos();
                g_oGamePiece.fScreenX = g_oGamePiece.fTargetScreenX;
                g_oGamePiece.fScreenY = g_oGamePiece.fTargetScreenY;
            }
            return true;
        }

        if( g_iCollisionFlags & COLLISION_FLAG_RIGHT )
        {
            iPieceX--;
        }
        else if( g_iCollisionFlags & COLLISION_FLAG_LEFT )
        {
            iPieceX++;
        }
        else
        {
            iPieceY--;
        }
    }

    // Roll back
    PieceRotateCounterClockwise( &g_oGamePiece );
    return false;
}

bool Game_PieceRotateCounterClockwise()
{
    int iPieceX = g_iGamePieceX;
    int iPieceY = g_iGamePieceY;
    PieceRotateCounterClockwise( &g_oGamePiece );

    int iMoveCnt = 3;
    while( iMoveCnt-- )
    {
        if( Game_TestGamePiece( iPieceX, iPieceY ) )
        {
            if( iPieceX != g_iGamePieceX || iPieceY != g_iGamePieceY )
            {
                g_iGamePieceX = iPieceX;
                g_iGamePieceY = iPieceY;
                Game_UpdateGamePieceScreenPos();
                g_oGamePiece.fScreenX = g_oGamePiece.fTargetScreenX;
                g_oGamePiece.fScreenY = g_oGamePiece.fTargetScreenY;
            }
            return true;
        }

        if( g_iCollisionFlags & COLLISION_FLAG_RIGHT )
        {
            iPieceX--;
        }
        if( g_iCollisionFlags & COLLISION_FLAG_LEFT )
        {
            iPieceX++;
        }
        if( g_iCollisionFlags & COLLISION_FLAG_BOTTOM )
        {
            iPieceY--;
        }
    }

    // Roll back
    PieceRotateClockwise( &g_oGamePiece );
    return false;
}

bool Game_TestGamePiece( i32 a_iX, i32 a_iY )
{
    g_iCollisionFlags = 0;

    i32 x, y;
    for( x = 0; x < 4; x++ )
    {
        for( y = 0; y < 4; y++ )
        {
            if( g_oGamePiece.aMap[x][y] )
            {
                int iPieceX = a_iX + x;
                int iPieceY = a_iY + y;

                if( iPieceX < 0 )
                {
                    g_iCollisionFlags |= COLLISION_FLAG_LEFT;
                    return false;
                }
                
                if( iPieceX >= WELL_WIDTH )
                {
                    g_iCollisionFlags |= COLLISION_FLAG_RIGHT;
                    return false;
                }
                
                if( iPieceY >= WELL_HEIGHT )
                {
                    g_iCollisionFlags |= COLLISION_FLAG_BOTTOM;
                    return false;
                }

                if( iPieceY >= 0 )
                {
                    if( g_aWell[ iPieceX ][ iPieceY ].bOccupied )
                    {
                        g_iCollisionFlags |= COLLISION_FLAG_PIECE;
                        return false;
                    }
                }
            }
        }
    }
    return true;
}

void Game_UpdateGamePieceScreenPos()
{
    g_oGamePiece.fTargetScreenX = itof( Well_GetScreenX( g_iGamePieceX ) );
    g_oGamePiece.fTargetScreenY = itof( Well_GetScreenY( g_iGamePieceY ) );
}

void Game_Render()
{
    int i;
    Screen_Clear( COLOR_BLUE );


    switch( g_eGameMode )
    {
    case GAME_MODE_WAIT:
        Well_RenderBackground(WELL_SCREEN_X, WELL_SCREEN_Y );
        Font_Print( "Tetris PSP", 200, 100, COLOR_BLUE );
        Font_Print( "By Erik Rounds", 200, 120, COLOR_BLUE );
        break;
    case GAME_MODE_PLAYING:
        Well_Render( WELL_SCREEN_X, WELL_SCREEN_Y );
        PieceRender( &g_oGamePiece );

        for( i = 0; i < NEXT_PIECE_COUNT; i++ )
        {
            PieceRender( &g_oNextPiece[ i ] );
        }
        break;
    case GAME_MODE_PAUSED:
        Well_RenderBackground(WELL_SCREEN_X, WELL_SCREEN_Y );
        Font_Print( "PAUSED!", 200, 100, COLOR_BLUE );
        Font_Print( "Press Start", 200, 120, COLOR_BLUE );
        break;
    }

    Game_RenderScore();

    Screen_WaitV();
    Screen_Swap();
}

char * my_itoa(i32 num, char *buf, int len, int base)
{
    bool bNeg = false;
    char *digits = "0123456789ABCDEF";
    char *fchar;

    buf[--len] = '\0';

    // first, fill in the number
    if(num < 0)
    {
        bNeg = true;
        num = -num;
    }
    else if(!num)
        buf[--len] = digits[0];
    while(num && len > 0)
    {
        buf[--len] = digits[num%base];
        num /= base;
    }
    // put in the sign (if needed)
    if(bNeg && len)
        buf[--len] = '-';

    fchar = &buf[len];

    // and pad with blanks
    while(len > 0)
        buf[--len] = ' ';

    return fchar;
}


void Game_RenderScore()
{
    char a[64];
    my_itoa( g_iScore, a, 64, 10 );

    Font_Print( "Score:", 340, 10, COLOR_WHITE );
    Font_Print( a, 360, 20, COLOR_WHITE );

    my_itoa( g_iLevel+1, a, 64, 10 );
    Font_Print( "Level:", 340, 60, COLOR_WHITE );
    Font_Print( a, 360, 70, COLOR_WHITE );

    int iNext = ((g_iLevel + 1) * LINES_PER_LEVEL) - g_iLinesCleared;
    my_itoa( iNext, a, 64, 10 );
    Font_Print( "Next:", 340, 110, COLOR_WHITE );
    Font_Print( a, 360, 120, COLOR_WHITE );
}
