#include "stdinc.h"

#include "piece.h"
#include "random.h"
#include "Screen.h"
#include "Cell.h"
#include "colors.h"

#define PIECE_TEMPLATE_COUNT 7

const pixel k_PieceColors[PIECE_TEMPLATE_COUNT] =
{
    COLOR_RED,
    COLOR_OLIVE,
    COLOR_CYAN,
    COLOR_ORANGE,
    COLOR_ROSYBROWN,
    COLOR_ORCHID,
    COLOR_TEAL
};

const u8 k_PieceTemplate[PIECE_TEMPLATE_COUNT][4][4] =
{
    {
        { 0, 1, 0, 0 },
        { 0, 1, 0, 0 },
        { 0, 1, 0, 0 },
        { 0, 1, 0, 0 },
    },
    {
        { 0, 0, 0, 0 },
        { 1, 1, 1, 0 },
        { 0, 0, 1, 0 },
        { 0, 0, 0, 0 },
    },
    {
        { 0, 0, 0, 0 },
        { 0, 1, 1, 1 },
        { 0, 1, 0, 0 },
        { 0, 0, 0, 0 },
    },
    {
        { 0, 1, 0, 0 },
        { 0, 1, 1, 0 },
        { 0, 1, 0, 0 },
        { 0, 0, 0, 0 },
    },
    {
        { 0, 0, 0, 0 },
        { 0, 1, 1, 0 },
        { 0, 1, 1, 0 },
        { 0, 0, 0, 0 },
    },
    {
        { 0, 0, 0, 0 },
        { 0, 1, 1, 0 },
        { 1, 1, 0, 0 },
        { 0, 0, 0, 0 },
    },
    {
        { 0, 0, 0, 0 },
        { 1, 1, 0, 0 },
        { 0, 1, 1, 0 },
        { 0, 0, 0, 0 },
    },

};

void PieceCreate( Piece* a_pThis )
{
    int iPieceType = RandInt()%PIECE_TEMPLATE_COUNT;

    MEMCPY( a_pThis->aMap, k_PieceTemplate[ iPieceType ], 16 );

    a_pThis->iColor = k_PieceColors[ iPieceType ];
}

void PieceUpdate( Piece* a_pThis )
{
    a_pThis->fScreenX -= a_pThis->fScreenX >> 2;
    a_pThis->fScreenX += a_pThis->fTargetScreenX >> 2;

    a_pThis->fScreenY -= a_pThis->fScreenY >> 2;
    a_pThis->fScreenY += a_pThis->fTargetScreenY >> 2;

    int iDistX = ftoi( a_pThis->fTargetScreenX - a_pThis->fScreenX );
    int iDistY = ftoi( a_pThis->fTargetScreenY - a_pThis->fScreenY );
    iDistX = ABS( iDistX );
    iDistY = ABS( iDistY );

    if( iDistX <= 2 ) {
        a_pThis->fScreenX = a_pThis->fTargetScreenX;
    }

    if( iDistY <= 2 ) {
        a_pThis->fScreenY = a_pThis->fTargetScreenY;
    }
}

void PieceRender( Piece* a_pThis )
{
    int x, y;

    int iScreenX = ftoi( a_pThis->fScreenX );
    int iScreenY = ftoi( a_pThis->fScreenY );

    for( x = 0; x < 4; x++ )
    {
        for( y = 0; y < 4; y++ )
        {
            if( a_pThis->aMap[ x ][ y ] ) {
                int iPieceX = iScreenX + x * CELL_WIDTH;
                int iPieceY = iScreenY + y * CELL_HEIGHT;
                int iHeight = CELL_HEIGHT;

                if( iPieceY < 0 ) {
                    iHeight += iPieceY;
                    iPieceY = 0;
                }

                if( iHeight > 0 )
                {
                    Screen_FillRect( iPieceX,
                        iPieceY,
                        CELL_WIDTH,
                        iHeight,
                        a_pThis->iColor );
                }
            }
        }
    }
}

void PieceRotateClockwise( Piece* a_pThis )
{
    u8 aTemp[4][4];
    int x, y;

    MEMCPY( aTemp, a_pThis->aMap, 16 );
    for( x = 0; x < 4; x++ )
    {
        for( y = 0; y < 4; y++ )
        {
            a_pThis->aMap[ x ][ y ] = aTemp[ 3 - y ][ x ];
        }
    }
}

void PieceRotateCounterClockwise( Piece* a_pThis )
{
    u8 aTemp[4][4];
    int x, y;
    MEMCPY( aTemp, a_pThis->aMap, 16 );
    for( x = 0; x < 4; x++ )
    {
        for( y = 0; y < 4; y++ )
        {
            a_pThis->aMap[ x ][ y ] = aTemp[ y ][ 3 - x ];
        }
    }
}
