#include "stdinc.h"
#include ".\well.h"

#include "Screen.h"
#include "colors.h"

void Well_RenderBackground( int a_iX, int a_iY )
{
    // Draw the well background
    Screen_FillRect( a_iX, a_iY, WELL_WIDTH_PIXELS, WELL_HEIGHT_PIXELS, COLOR_WHITE );

}

void Well_Render( int a_iX, int a_iY )
{
    int x,y;

    Well_RenderBackground(a_iX,a_iY);

    for( y = 0; y < WELL_HEIGHT; y++ )
    {
        for( x = 0; x < WELL_WIDTH; x++ )
        {
            Cell* pCell = &g_aWell[x][y];
            if( pCell->bOccupied )
            {
                Screen_FillRect( Well_GetScreenX( x ),
                                Well_GetScreenY( y ),
                                CELL_WIDTH,
                                CELL_HEIGHT,
                                pCell->iColor );
            }
        }
    }
}

void Well_Update()
{
}

void Well_Init()
{
    MEMSET( g_aWell, 0, sizeof( g_aWell ) );
}

void Well_Deinit()
{
}

i32 Well_GetScreenX( int a_iCellX )
{
    return WELL_SCREEN_X + a_iCellX * CELL_WIDTH;
}

i32 Well_GetScreenY( int a_iCellY )
{
    return WELL_SCREEN_Y + a_iCellY * CELL_HEIGHT;
}

int Well_CheckForCompleteLines()
{
    int x, y;
    int iCompleteLines = 0;

    for( y = 0; y < WELL_HEIGHT; y++ )
    {
        bool bCompleteLine = true;
        for( x = 0; x < WELL_WIDTH; x++ )
        {
            if( !g_aWell[ x ][ y ].bOccupied )
            {
                bCompleteLine = false;
                break;
            }
        }

        if( bCompleteLine )
        {
            iCompleteLines++;

            Well_RemoveLine( y );
        }
    }

    return iCompleteLines;
}

void Well_RemoveLine( int a_iY )
{
    int x, y;
    for( y = a_iY; y >= 1; y-- )
    {
        for( x = 0; x < WELL_WIDTH; x++ )
        {
            g_aWell[ x ][ y ].bOccupied = g_aWell[ x ][ y - 1 ].bOccupied;
            g_aWell[ x ][ y ].iColor = g_aWell[ x ][ y - 1 ].iColor;
        }
    }

    // Clear the top line
    for( x = 0; x < WELL_WIDTH; x++ )
    {
        g_aWell[ x ][ 0 ].bOccupied = false;
    }
}

Cell g_aWell[WELL_WIDTH][WELL_HEIGHT];
